unit AddEditGUI;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  EditBtn, vTypes, vAddEdit, vCourses, vTableData, vStringData, LCLtype;

type

  { TfrmAddEditGUI }

  TfrmAddEditGUI = class(TForm)
    btnCancel: TButton;
    btnSave: TButton;
    procedure btnCancelClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
  private
    { private declarations }
    TableData: TvTableData;
    StringData: TvStringData;
    Fields: vFieldArrayType;
    AddEdit: TvAddEdit;
    Courses: TvCourses;
    NumberOfAEControls: integer;
    // = True of adding a new record
    NewRecord: boolean;
    // = True if don't close the form in 'onclose' event (set in btnCancel and btnSave)
    DontClose: boolean;
    // Used in DataChanged property
    FDataChanged: boolean;
    // Checks on form create to see if we are adding or editing a record
    procedure CheckForNewRecord;
    // Used in DataChanged property
    procedure SetDataChanged(const AValue: boolean);
  public
    { public declarations }
    constructor Create(Aowner: Tcomponent; Tdata: TvTableData; Corses: TvCourses; StrData: TvStringData;
      Flds: vFieldArrayType; Table: string = '');  reintroduce;
    destructor Destroy; override;
    // Course name and description and other course details used by vAddEdit when adding a new course
    function GetCourseName(out OtherDetails: TvUpdateCourseType):string;
    // Called by MainGUI to check if Data has been changed.
    property DataChanged:boolean read FdataChanged write SetDataChanged;
  end;

implementation

{$R *.lfm}

{ TfrmAddEditGUI }

procedure TfrmAddEditGUI.btnCancelClick(Sender: TObject);
begin
  DontClose := True;
  if AddEdit.DataChanged then begin
    if Application.MessageBox('Data has not been saved. Are you sure you want to close?',
    'Warning -- Unsaved Data', MB_ICONQUESTION + MB_YESNO + MB_DEFBUTTON3) = idYes then
      DontClose := False;
  end
  else
    DontClose := False;
end;

procedure TfrmAddEditGUI.btnSaveClick(Sender: TObject);
var
  ErrorMessage: string;
begin
  DontClose := True;
  if not AddEdit.SaveData(ErrorMessage) then
    MessageDlg('Error on Saving.', ErrorMessage, mtWarning, [mbOK], 0)
  else begin
    DontClose := False;
    DataChanged := True;
  end;
end;

procedure TfrmAddEditGUI.FormCloseQuery(Sender: TObject; var CanClose: boolean);
begin
  CanClose := not DontClose;
end;

procedure TfrmAddEditGUI.CheckForNewRecord;
begin
  if Fields[0].Value = '' then begin
    NewRecord := True;
    Caption := 'New Entry';
  end
  else begin
    NewRecord := False;
    Caption := 'Edit Entry';
  end;
end;

procedure TfrmAddEditGUI.SetDataChanged(const AValue: boolean);
begin
  FdataChanged:=AValue;
end;

constructor TfrmAddEditGUI.Create(Aowner: Tcomponent;  Tdata: TvTableData; Corses: TvCourses;  StrData: TvStringData;
  Flds: vFieldArrayType; Table: string = '');
begin
  TableData          := Tdata;
  StringData         := StrData;
  Fields             := Flds;
  Courses            := Corses;
  // '-1' because getNumberOfFields does not include the ID field while AddEdit does.
  NumberOfAEcontrols := TableData.GetNumberOfFields(Table,False) - 1;
  DontClose          := False;
  DataChanged        := False;
  inherited Create(Aowner);
  AddEdit := TvAddEdit.Create(TableData, Courses, StringData, self, Fields, NumberOfAEcontrols, Table);
  CheckForNewRecord;
end;

destructor TfrmAddEditGUI.Destroy;
begin
  AddEdit.Destroy;
  inherited Destroy;
end;

function TfrmAddEditGUI.GetCourseName(out OtherDetails: TvUpdateCourseType): string;
begin
  Result := AddEdit.GetCourseName(OtherDetails);
end;

end.

