unit EditTablesGUI;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ComCtrls, vSqlite3, vTableData, vINI, vTypes, vUtils, vCourses;

type

  { TfrmEditTablesGUI }

  TfrmEditTablesGUI = class(TForm)
    btnNewField: TButton;
    btnSave: TButton;
    btnCancel: TButton;
    btnNewTable: TButton;
    chkFieldDefaultX: TCheckBox;
    chkTableGraphed: TCheckBox;
    chkFieldHidden: TCheckBox;
    chkFieldDefaultY: TCheckBox;
    chkTableHidden: TCheckBox;
    cmbFieldType: TComboBox;
    cmbFieldDefaultValue: TComboBox;
    edtFieldDefaultValue: TEdit;
    edtNewTableName: TEdit;
    edtNewFieldName: TEdit;
    edtUFtableName: TEdit;
    edtUFfieldName: TEdit;
    gbxFieldProperties: TGroupBox;
    gbxTables: TGroupBox;
    gbxTableProperties: TGroupBox;
    gbxFields: TGroupBox;
    lblFieldDefaultValue: TLabel;
    lblFieldType: TLabel;
    lblUFtableName: TLabel;
    lblUFfieldName: TLabel;
    lbxTableNames: TListBox;
    lbxFieldNames: TListBox;
    updTableNames: TUpDown;
    updFieldNames: TUpDown;
    procedure btnCancelClick(Sender: TObject);
    procedure btnSaveClick(Sender: TObject);
    procedure cmbFieldTypeChange(Sender: TObject);
    procedure FormCloseQuery(Sender: TObject; var CanClose: boolean);
    procedure lbxFieldNamesSelectionChange(Sender: TObject; User: boolean);
    procedure lbxTableNamesSelectionChange(Sender: TObject; User: boolean);
    procedure updFieldNamesClick(Sender: TObject; Button: TUDBtnType);
    procedure updTableNamesClick(Sender: TObject; Button: TUDBtnType);
    procedure FillFieldListBox;
    procedure OpenDependantUnits(const DBPath, DataPath: string);
    procedure CloseDependantUnits;
  private
    { private declarations }
    Sqlite3: TvSqlite3;
    vINI: TvINI;
    TableData: TvTableData;
    Courses: TvCourses;
    TableInfo: VTableInfoArrayType;
    TableIndex, FieldIndex: integer;
    // The order to display tables. each element is the index to TableInfo.
    TableOrder: Array[0..MaxTables] of integer;
    FieldOrder: Array[0..MaxTables,0..MaxFields] of integer;
    // = True if user in btnCancel does not wnt to close the form
    DontClose: boolean;
    DataChanged: boolean;
    function GetLastField: integer;
    property LastField: integer read GetLastField;
    procedure LoadControls;
    procedure SwapTableNames(const ndx1, ndx2: integer);
    procedure SwapFieldNames(const ndx1, ndx2: integer);
    procedure SaveCurrentTable;
    procedure SaveCurrentField;
    function MovingDescriptionField(const MovePrevious: boolean): boolean;
    function MovingIDField(const MovePrevious: boolean): boolean;
    function IsLegalFieldMove(const MovePrevious: boolean): boolean;
    procedure RemoveAllDefaultY;
    procedure RemoveAllDefaultX;
    procedure RemoveSimilarFieldTypes(const FieldType: vFieldType);
    procedure ShowCourseComboBox(Display: boolean);
    procedure FillCourseComboBox;
    procedure CourseComboBoxOnChange(Sender: TObject);
    procedure SaveTableOrder;
    procedure SaveFieldOrder(const Tindex: integer);
  public
    { public declarations }
    constructor Create(const DBPath, DataPath: string); reintroduce;
    destructor Destroy(); override;
  end; 

var
  frmEditTablesGUI: TfrmEditTablesGUI;

implementation

{$R *.lfm}

{ TfrmEditTablesGUI }

procedure TfrmEditTablesGUI.btnSaveClick(Sender: TObject);
begin
  SaveCurrentTable;
  SaveCurrentField;
  vINI.SaveTableInfo(TableInfo);
  SaveTableOrder;
  DontClose := False;
 end;

procedure TfrmEditTablesGUI.cmbFieldTypeChange(Sender: TObject);
begin
  if (CompareText(cmbFieldType.Text,'Primary Key') = 0) and (lbxFieldNames.ItemIndex <> 0) then begin
    MessageDlg('Error setting Primary Key', 'Only the first field may be the Primary Key',mtError, [mbOK], 0);
    cmbFieldType.ItemIndex := cmbFieldType.Items.IndexOf('Nothing special');
  end
  else if (CompareText(cmbFieldType.Text,'Description Field') = 0) and
          (lbxFieldNames.ItemIndex <> lbxFieldNames.Items.Count-1) then begin
    if MessageDlg('Warning: Description Field', 'Only the last field may be the Description Field.v' +
      'Would you like this field moved to the last field?', mtWarning, mbYesNo, 0) = mrYes then
        SwapFieldNames(lbxFieldNames.ItemIndex,lbxFieldNames.Items.Count-1)
    else
      cmbFieldType.ItemIndex := cmbFieldType.Items.IndexOf('Nothing special');
  end;
end;

procedure TfrmEditTablesGUI.FormCloseQuery(Sender: TObject;
  var CanClose: boolean);
begin
  CanClose := not DontClose;
end;

procedure TfrmEditTablesGUI.btnCancelClick(Sender: TObject);
begin
  SaveCurrentTable;
  SaveCurrentField;
  DontClose := False;
  if DataChanged and (MessageDlg('Warning -- Data has changed',
      'There is unsaved changes. Do you wish to close without saving?',
      mtWarning, mbYesNo, 0) = mrNo) then DontClose := True;
end;

procedure TfrmEditTablesGUI.lbxFieldNamesSelectionChange(Sender: TObject;
  User: boolean);
begin
  if User then
    SaveCurrentField;
  ShowCourseComboBox(False);
  FieldIndex                 := FieldOrder[TableIndex,lbxFieldNames.ItemIndex];
  gbxFieldProperties.Caption := 'Properties of ''' + TableInfo[TableIndex].FieldInfo[FieldIndex].FN + '''';
  chkFieldHidden.Checked     := TableInfo[TableIndex].FieldInfo[FieldIndex].Hidden;
  edtUFfieldName.Text        := TableInfo[TableIndex].FieldInfo[FieldIndex].UFN;
  if integer(lbxFieldNames.items.Objects[lbxFieldNames.ItemIndex]) = 0 then
    cmbFieldType.ItemIndex   := cmbFieldType.Items.Count-1
  else
    cmbFieldType.ItemIndex   := ord(TableInfo[TableIndex].FieldInfo[FieldIndex].FT);
  chkFieldDefaultY.Checked   := TableInfo[TableIndex].FieldInfo[FieldIndex].Y;
  chkFieldDefaultX.Checked   := TableInfo[TableIndex].FieldInfo[FieldIndex].X;
  edtFieldDefaultValue.Text  := TableInfo[TableIndex].FieldInfo[FieldIndex].DefaultValue;
  if TableInfo[TableIndex].FieldInfo[FieldIndex].FT = vCourse then
    ShowCourseComboBox(True);
  if lbxFieldNames.ItemIndex = 0 then
    cmbFieldType.Enabled := False
  else
    cmbFieldType.Enabled := True;
  Unused(User);
end;

procedure TfrmEditTablesGUI.lbxTableNamesSelectionChange(Sender: TObject;
  User: boolean);
begin
  If User then begin
    SaveCurrentTable;
    SaveCurrentField;
  end;
  TableIndex                 := TableOrder[lbxTableNames.ItemIndex];
  gbxTableProperties.Caption := 'Properties of ''' + TableInfo[TableIndex].TN + '''';
  edtUFtableName.Text        := TableInfo[TableIndex].UFN;
  chkTableHidden.checked     := TableInfo[TableIndex].Hidden;
  chkTableGraphed.Checked    := not TableInfo[TableIndex].NotGraphed;
  gbxFields.Caption          := 'Fields of the Table ''' + TableInfo[TableIndex].TN + '''';
  FillFieldListBox;
  Unused(User);
end;

procedure TfrmEditTablesGUI.updFieldNamesClick(Sender: TObject;
  Button: TUDBtnType);
begin
  case Button of
    btPrev: if (lbxFieldNames.Items.Count <> lbxFieldNames.ItemIndex + 1) and
                IsLegalFieldMove(Button = btPrev) then
              SwapFieldNames(lbxFieldNames.ItemIndex, lbxFieldNames.ItemIndex+1);
    btNext: if (lbxFieldNames.Itemindex > 0) and IsLegalFieldMove(Button = btPrev) then
              SwapFieldNames(lbxFieldNames.ItemIndex, lbxFieldNames.ItemIndex-1);
  end;
end;

procedure TfrmEditTablesGUI.updTableNamesClick(Sender: TObject;
  Button: TUDBtnType);
begin
  case Button of
    btPrev: if lbxTableNames.Items.Count <> lbxTableNames.ItemIndex + 1 then
              SwapTableNames(lbxTableNames.ItemIndex, lbxTableNames.ItemIndex+1);
    btNext: if lbxTableNames.Itemindex <> 0 then
              SwapTableNames(lbxTableNames.ItemIndex, lbxTableNames.ItemIndex-1);
  end;
end;

procedure TfrmEditTablesGUI.FillFieldListBox;
var
  FieldNdx: integer;
begin
  lbxFieldNames.items.Clear;
  for FieldNdx := 0 to TableInfo[TableIndex].NF -1 do begin
    FieldIndex := FieldOrder[TableIndex, FieldNdx];
    lbxFieldNames.Items.AddObject(TableInfo[TableIndex].FieldInfo[FieldIndex].FN, TObject(FieldIndex));
  end;
  lbxFieldNames.ItemIndex    := 0;
  // Required to generate a selection change event in lbxFieldNames
  // [at least on the Mac...not tested in Windows.]
  lbxFieldNames.ItemIndex    := 1;
  lbxFieldNames.ItemIndex    := 0;
end;

procedure TfrmEditTablesGUI.LoadControls;
var
  i, j: integer;
begin
  TableInfo := TableData.AllTableInfo;
  for i := 0 to High(TableInfo) do
    if TableInfo[i].NF <> 0 then begin
      TableOrder[i] := i;
      for j := 0 to TableInfo[i].NF - 1 do
        FieldOrder[i,j] := j;
    end
    else
      TableOrder[i] := -1;
  TableData.LoadTableNames(lbxTableNames);
  FillCourseComboBox;
end;

procedure TfrmEditTablesGUI.SwapTableNames(const ndx1, ndx2: integer);
var
  i: integer;
begin
  i := TableOrder[ndx1];
  TableOrder[ndx1] := Tableorder[ndx2];
  TableOrder[ndx2] := i;
  lbxTableNames.Items.Exchange(ndx1, ndx2);
  lbxTableNames.ItemIndex := ndx2;
end;

procedure TfrmEditTablesGUI.SwapFieldNames(const ndx1, ndx2: integer);
var
  i: integer;
begin
{  showmessage('ndx1='+IntToStr(ndx1)+', ndx2='+IntToStr(ndx2)+LineEnding+
  'FieldOrder of ndx1='+inttostr(FieldOrder[TableIndex,ndx1])+
  ', FieldOrder of ndx2='+inttostr(FieldOrder[TableIndex,ndx2])); }
  i := FieldOrder[TableIndex, ndx1];
  FieldOrder[TableIndex, ndx1] := FieldOrder[TableIndex,ndx2];
  FieldOrder[TableIndex, ndx2] := i;
  lbxFieldNames.Items.Exchange(ndx1, ndx2);
  lbxFieldNames.ItemIndex := ndx2;
end;

procedure TfrmEditTablesGUI.SaveCurrentTable;
begin
  if (TableInfo[TableIndex].NotGraphed <> not chkTableGraphed.Checked) or
     (TableInfo[TableIndex].Hidden <> chkTableHidden.Checked) or
     (TableInfo[TableIndex].UFN    <> edtUFtableName.Text) then
       DataChanged := True;
  TableInfo[TableIndex].NotGraphed := not chkTableGraphed.Checked;
  TableInfo[TableIndex].Hidden     := chkTableHidden.Checked;
  TableInfo[TableIndex].UFN        := edtUFtableName.Text;
end;

procedure TfrmEditTablesGUI.SaveCurrentField;
begin
  if (TableInfo[TableIndex].FieldInfo[FieldIndex].DefaultValue <> edtFieldDefaultValue.Text) or
     (TableInfo[TableIndex].FieldInfo[FieldIndex].UFN <> edtUFfieldName.Text) or
     (TableInfo[TableIndex].FieldInfo[FieldIndex].Hidden  <> chkFieldHidden.Checked) or
     ((integer(lbxFieldNames.items.Objects[lbxFieldNames.ItemIndex]) <> 0) and
      (TableInfo[TableIndex].FieldInfo[FieldIndex].FT <> vFieldType(cmbFieldType.ItemIndex))) then
        DataChanged := True;
  if chkFieldDefaultY.Checked then RemoveAllDefaultY;
  if chkFieldDefaultX.Checked then RemoveAllDefaultX;
  TableInfo[TableIndex].FieldInfo[FieldIndex].Y            := chkFieldDefaultY.Checked;
  TableInfo[TableIndex].FieldInfo[FieldIndex].X            := chkFieldDefaultX.Checked;
  TableInfo[TableIndex].FieldInfo[FieldIndex].DefaultValue := edtFieldDefaultValue.Text;
  TableInfo[TableIndex].FieldInfo[FieldIndex].UFN          := edtUFfieldName.Text;
  TableInfo[TableIndex].FieldInfo[FieldIndex].Hidden       := chkFieldHidden.Checked;
  if integer(lbxFieldNames.items.Objects[lbxFieldNames.ItemIndex]) <> 0 then begin
    RemoveSimilarFieldTypes(vFieldType(cmbFieldType.ItemIndex));
    TableInfo[TableIndex].FieldInfo[FieldIndex].FT         := vFieldType(cmbFieldType.ItemIndex);
  end;
end;

function TfrmEditTablesGUI.MovingDescriptionField(const MovePrevious: boolean): boolean;
begin
  case MovePrevious of
    True:  Result := ((lbxFieldNames.Items.Count-2) = lbxFieldNames.ItemIndex) and
                     (TableInfo[TableIndex].FieldInfo[LastField].FT = vDescription);
    False: Result := ((lbxFieldNames.Items.Count-1) = lbxFieldNames.ItemIndex) and
                     (TableInfo[TableIndex].FieldInfo[LastField].FT = vDescription);
  end;
end;

function TfrmEditTablesGUI.MovingIDField(const MovePrevious: boolean): boolean;
begin
  case MovePrevious of
    True:  Result := (lbxFieldNames.ItemIndex = 0);
    False: Result := (lbxFieldNames.Itemindex = 1);
  end;
end;

function TfrmEditTablesGUI.IsLegalFieldMove(const MovePrevious: boolean): boolean;
var
  ErrorMsg: string = '';
begin
  Result := True;
  if MovingDescriptionField(MovePrevious) then
    ErrorMsg := 'The Description field must be the last field.'
  else if MovingIDField(MovePrevious) then
    ErrorMsg := 'The ID field (primary key) must be the first field.';
  if ErrorMsg <> '' then begin
    Result := False;
    MessageDlg('Error -- illegal change of order',ErrorMsg,mtError, [mbOK], 0);
  end;
end;

procedure TfrmEditTablesGUI.RemoveAllDefaultY;
var
  i: integer;
begin
  for i := 0 to TableInfo[TableIndex].NF -1 do
    TableInfo[TableIndex].FieldInfo[i].Y := False;
end;

procedure TfrmEditTablesGUI.RemoveAllDefaultX;
var
  i: integer;
begin
  for i := 0 to TableInfo[TableIndex].NF -1 do
    TableInfo[TableIndex].FieldInfo[i].X := False;
end;

procedure TfrmEditTablesGUI.RemoveSimilarFieldTypes(const FieldType: vFieldType);
var
  i: integer;
begin
 for i := 0 to TableInfo[TableIndex].NF -1 do
   if TableInfo[TableIndex].FieldInfo[i].FT = FieldType then
     TableInfo[TableIndex].FieldInfo[i].FT := vOther;
end;

procedure TfrmEditTablesGUI.ShowCourseComboBox(Display: boolean);
begin
  edtFieldDefaultValue.Visible := not Display;
  cmbFieldDefaultValue.Visible := Display;
  if Display then
    cmbFieldDefaultValue.ItemIndex := cmbFieldDefaultValue.Items.IndexOf((edtFieldDefaultValue.Text));
end;

procedure TfrmEditTablesGUI.FillCourseComboBox;
begin
  Courses.RegisterComboBox(cmbFieldDefaultValue);
  cmbFieldDefaultValue.OnChange := @CourseComboBoxOnChange
end;

procedure TfrmEditTablesGUI.CourseComboBoxOnChange(Sender: TObject);
begin
  edtFieldDefaultValue.Text := cmbFieldDefaultValue.Text;
end;

procedure TfrmEditTablesGUI.SaveTableOrder;
var
  Torder: TstringList;
  i: integer;
begin
  Torder := TstringList.Create;
  for i := 0 to MaxTables do begin
    if TableOrder[i] = -1 then break;
    Torder.Add(Tableinfo[TableOrder[i]].TN);
    SaveFieldOrder(TableOrder[i]);
  end;
  vINI.TableOrder := Torder;
  Torder.Destroy;
end;

procedure TfrmEditTablesGUI.SaveFieldOrder(const Tindex: integer);
var
  FieldList: TstringList;
  i: integer;
begin
  FieldList := TstringList.Create;
  for i := 0 to TableInfo[Tindex].NF - 1 do begin
    FieldList.Add(TableInfo[Tindex].FieldInfo[FieldOrder[Tindex,i]].FN);
  end;
  vINI.SetFieldOrder(TableInfo[Tindex].TN, FieldList);
  FieldList.Destroy;
end;

destructor TfrmEditTablesGUI.Destroy();
begin
  Courses.UnRegisterComboBox(cmbFieldDefaultValue);
  CloseDependantUnits;
  inherited Destroy;
end;

constructor TfrmEditTablesGUI.Create(const DBPath, DataPath: string);
begin
  inherited Create(nil);
  OpenDependantUnits(DBpath, DataPath);
  LoadControls;
  DataChanged := False;
end;

procedure TfrmEditTablesGUI.OpenDependantUnits(const DBPath, DataPath: string);
begin
  Sqlite3    := Tvsqlite3.Create(DBpath);
  vINI       := TvINI.Create(DataPath);
  TableData  := TvTableData.Create(vINI, Sqlite3);
  Courses    := TvCourses.Create(Sqlite3);
end;

procedure TfrmEditTablesGUI.CloseDependantUnits;
begin
  Courses.Destroy;
  TableData.Destroy;
  vINI.Destroy;
  Sqlite3.Destroy;
end;

function TfrmEditTablesGUI.GetLastField: integer;
begin
  Result := FieldOrder[TableIndex, TableInfo[TableIndex].NF-1];
end;

end.

