unit vAddEdit;

{$mode objfpc}{$H+}

////////////////////////////////////////////////////////////////////////
////       Unit providing 'Adding/Editing a field' operations       ////
////////////////////////////////////////////////////////////////////////

interface

uses
  Classes, Forms, Controls, Dialogs, ExtCtrls, StdCtrls, EditBtn, vTableData, vCourses, vStringData,
  vTypes, LCLtype, vUtils;

type

  // Used by variable AE for displaying labels and edit controls for adding new data in frmAddEdit
  AEarrayType = array [0..MaxFields] of TLabeledEdit;
  // Used by OtherInfo for storing Course info (eg name, description)
  TvUpdateCourseType = array[0..2] of string;
  // Function definition for calling from 'Iterate'. This function performs an action on an AE control
  TvAEoperation = function(const currField: integer; const currAE: integer; const FieldType:vFieldType): boolean of object;

const
    // Constants for drawing AE controls
    InitialLeft            = 220;
    InitialTop             = 30;
    DistanceToSecondColumn = 371;
type

  { TvAddEdit }

  TvAddEdit = class
    private
      TableData: TvTableData;
      Courses: TvCourses;
      StringData: TvStringData;
      TabPage: Tform;
      Fields: vFieldArrayType;
      CurrentTable: string;
      ErrorMsg: string;
      // LabelEdit controls used for adding/editing table values in tabAddEdit
      AE: AEarrayType;
      DteDate: TDateEdit;
      cmbCourses: TComboBox;
      lblDate: TLabel;
      lblCourse: Tlabel;
      btnNew: TButton;
      btnGo: TButton;
      // Used by NumberOfAEcontrols property
      FNumberOfAEcontrols: integer;
      // Course details, used when adding a new course:
      // OtherDetails[0] = Distance
      // OtherDetails[1] = URL
      // OtherDetails[2] = Description
      OtherDetails: TvUpdateCourseType;
      // Used by the NumberofAEcontrols property
      procedure SetNumberOfAEcontrols(const AValue: integer);
      // Create the Controls for user input
      procedure CreateControls;
      procedure CreateLabelledEditControls;
      procedure CreateOtherControls;
      // Reset values of the AE controls
      procedure ClearAEcontrols;
      // Fill the labels and comboboxes and date controls with info for the current table
      procedure InitAEforCurrentTable;
      // Ensure the 6th AE control is used for the Description field
      function SetDescriptionAEcontrol(const CurrAE: integer; const i:integer; const FieldType: vFieldType): integer;
      // Set the width of the AE control
      function SetAEcontrolWidth(const FieldType: vFieldType):integer;
      // Configure AE controls for 'special' fieldtypes (eg vDate, vCourse)
      procedure ConfigureSpecialAEcontrols(const CurrAE: integer; const  FieldType: vFieldType;
        const uffieldName: string);
      // Set the text of the ith AE control
      procedure SetText(i: integer; Text: string; OnlyIfEmpty: boolean);
      // Get the text from the ith AE control
      function GetText(i: integer):string;
      // Ensure AE controls contain the data from the comboboxes, date controls, etc.
      procedure UpdateAEfromSpecialControls;
      // Action when clicking on btnNew ('New Course') - ie a new addEditGUI to enter a new course
      procedure btnNewClick(Sender: Tobject);
      // Check that the entered data is in 'dd/mm/yyyy' format
      function ValidateDate(out ErrorMessage: string; const Text: string): boolean;
      // Check that the entered data is in 'mm:ss' format
      function ValidateSeconds(out ErrorMessage: string; const Text: string): boolean;
      // When the Course changes, fill the description and distance fields on the data entry form
      procedure CoursesOnChange(Sender: Tobject);
      // Iterate through the AE controls calling the called function to operate on each AE control
      // if the function returns false, then stop iterating.
      function  AEiterate(AEoperation: TvAEoperation):boolean;
      function IterationOfCoursesOnChange(const currField: integer; const CurrAE: integer; const FieldType:vFieldType):boolean;
      function IterationOfInitAEforCurrentTable(const currField: integer; const CurrAE: integer; const FieldType: vFieldType): boolean;
      function IterationOfFillAEcontrolsWithData(const currField: integer; const CurrAE: integer; const FieldType:vFieldType):boolean;
      function IterationOfDataChanged(const currField: integer; const CurrAE: integer; const FieldType:vFieldType):boolean;
      function IterationOfSaveData(const currField: integer; const CurrAE: integer; const FieldType:vFieldType):boolean;
      function IterationOfValidateData(const currField: integer; const CurrAE: integer; const FieldType:vFieldType):boolean;
      // Check all fields for valid data
      function ValidData: boolean;
      // Map the course
      procedure btnGoClick(Sender: TObject);
   public
      constructor Create(TData: TvTableData; Corses: TvCourses; StrData: TvStringData;
        Form: Tform; Feelds: vFieldArrayType; NumberOfAEs: integer; Table: string = '');
      destructor Destroy; override;
      // Number of Controls in the array 'AE' currently being used
      property NumberOfAEcontrols: integer read FNumberOfAEcontrols write SetNumberOfAEcontrols;
      procedure FillAEcontrolsWithData;
      function DataChanged:boolean;
      function  SaveData(out ErrorMessage: string): boolean;
      function GetCourseName(out OtherInfo:TvUpdateCourseType): string;
  end;

implementation

uses SysUtils, StrUtils, AddEditGUI;

{ TvAddEdit }

procedure TvAddEdit.SetNumberOfAEcontrols(const AValue: integer);
begin
  FNumberOfAEcontrols:=AValue;
end;

procedure TvAddEdit.CreateControls;
begin
  CreateLabelledEditControls;
  CreateOtherControls;
end;

procedure TvAddEdit.CreateLabelledEditControls;
var
  i, currLeft, currTop: integer;
begin
  currLeft := InitialLeft;
  currTop  := InitialTop;
  for i := 0 to (MaxFields) do
  begin
    AE[i]               := TLabeledEdit.Create(nil);
    AE[i].Parent        := TabPage;
    AE[i].Visible       := False;
    AE[i].Left          := currLeft;
    AE[i].Top           := currTop;
    AE[i].LabelPosition := lpLeft;
    AE[i].TabOrder      := i;
    currTop             := currTop + 36;
    if i = 6 then
    begin
      currLeft := currLeft + DistanceToSecondColumn;
      currTop  := 20;
    end;
  end;
end;

procedure TvAddEdit.CreateOtherControls;
begin
  dteDate         := TDateEdit.Create(nil);
  with dteDate do begin
    Parent       := TabPage;
    Visible      := False;
    DefaultToday := True;
  end;

  cmbCourses         := TComboBox.Create(nil);
  with cmbCourses do begin
    Parent   := TabPage;
    Visible  := False;
    Sorted   := True;
    ShowHint := True;
    Style    := csDropDownList;
    OnSelect := @CoursesOnChange;
  end;

  lblDate          := TLabel.Create(nil);
  with lblDate do begin
    Width    := 40;
    Caption  := 'Date';
    AutoSize := True;
    Parent   := TabPage;
    Visible  := False;
  end;

  lblCourse          := Tlabel.Create(nil);
  with lblCourse do begin
    Width    := 45;
    Caption  := 'Course';
    AutoSize := True;
    Visible  := False;
    Parent   := TabPage;
  end;

  btnNew          := TButton.Create(nil);
  with btnNew do begin
    Caption  := 'New';
    Width    := 50;
    Height   := AE[0].Height;
    Visible  := False;
    Parent   := TabPage;
    OnClick  := @btnNewClick;
  end;

  btnGo         := TButton.Create(nil);
  with btnGo do begin
    Caption := 'Go';
    Visible := False;
    Parent  := TabPage;
    onClick := @btnGoClick;
  end;
end;

function TvAddEdit.IterationOfInitAEforCurrentTable(const currField: integer; const CurrAE: integer; const FieldType: vFieldType): boolean;
var
  UFfieldName: string;
begin
  Result := True;
  UFfieldName := TableData.GetithUFfieldName(currField, CurrentTable);
  AE[CurrAE].EditLabel.Caption := UFfieldName;
  AE[CurrAE].Width             := SetAEcontrolWidth(FieldType);
  AE[CurrAE].Visible           := True;
  AE[CurrAE].Tag               := CurrAE;
  AE[CurrAE].TabOrder          := CurrAE;
  ConfigureSpecialAEcontrols(CurrAE, FieldType, UFfieldName);
end;

procedure TvAddEdit.InitAEforCurrentTable;
begin
  ClearAEcontrols;
  AEiterate(@IterationOfInitAEforCurrentTable);
end;

procedure TvAddEdit.ClearAEcontrols;
var
  i: integer;
begin
  for i := 0 to MaxFields do
  begin
    AE[i].EditLabel.Caption := '';
    AE[i].Text              := '';
    AE[i].TabOrder          := i;
    AE[i].Tag               := -1;
    AE[i].Visible           := False;
  end;
  dteDate.Visible    := False;
  lblDate.Visible    := False;
  cmbCourses.Visible := False;
  lblCourse.Visible  := False;
  btnNew.Visible     := False;
  btnGo.Visible      := False;
end;

function TvAddEdit.SetDescriptionAEcontrol(const CurrAE: integer; const i: integer; const FieldType: vFieldType): integer;
begin
  Result := CurrAE;
  if (FieldType = vDescription) and (i > 6) then
    Result := 6
  else if CurrAE = 6 then
    Result := 7;
end;

function TvAddEdit.SetAEcontrolWidth(const FieldType: vFieldType): integer;
begin
  if FieldType = vDescription then
    Result := 470
  else if FieldType = vURL then
    Result := 250
  else
    Result := 100;
end;

procedure TvAddEdit.ConfigureSpecialAEcontrols(const CurrAE: integer;
  const FieldType: vFieldType; const uffieldName: string);
begin
  case FieldType of
    vCourse: begin
               AE[CurrAE].Visible  := False;
               lblCourse.Caption   := UFfieldName;
               cmbCourses.Left     := AE[CurrAE].Left;
               cmbCourses.Top      := AE[CurrAE].top;
               cmbCourses.Tag      := CurrAE;
               cmbCourses.TabOrder := CurrAE;
//               {$ifdef Darwin}
               cmbCourses.Width       := 170;
//               {$endif}
               Courses.RegisterComboBox(cmbCourses);
               lblCourse.Visible   := True;
               lblCourse.Height    := cmbCourses.Height;
               lblCourse.Left      := cmbCourses.Left - lblCourse.Width - 2;
               lblCourse.Top       := cmbCourses.Top + 2;
               cmbCourses.Visible  := True;
               btnNew.Visible      := True;
               btnNew.Top          := lblCourse.Top;
               btnNew.Left         := cmbCourses.Left + cmbCourses.Width + 2;
             end;
    vDate: begin
             AE[CurrAE].Visible  := False;
             dteDate.Visible     := True;
             dteDate.Left        := AE[CurrAE].Left;
             dteDate.Top         := AE[CurrAE].top;
             dteDate.Text        := AE[CurrAE].Text;
             dteDate.Tag         := CurrAE;
             dteDate.TabOrder    := CurrAE;
             {$ifdef Darwin}
             dteDate.Width       := 85;
             {$endif}
             lblDate.Visible     := True;
             lblDate.Left        := AE[CurrAE].Left - lblDate.Width - 2;
             lblDate.Top         := AE[CurrAE].Top + 2;
           end;
    vURL: begin
            AE[CurrAE].TabOrder := CurrAE;
            btnGo.Visible       := True;
            btnGo.Top           := AE[CurrAE].Top;
            btnGo.Left          := AE[CurrAE].Left + AE[CurrAE].Width + 2;
            btnGo.Tag           := CurrAE;
          end;
     end;
end;

constructor TvAddEdit.Create(TData: TvTableData; Corses: TvCourses; StrData: TvStringData;
  Form: Tform; Feelds: vFieldArrayType; NumberOfAEs: integer; Table: string = '');
begin
  TableData          := Tdata;
  Courses            := Corses;
  StringData         := StrData;
  TabPage            := Form;
  Fields             := Feelds;
  NumberOfAEcontrols := NumberOfAEs;
  CurrentTable       := Table;
  CreateControls;
  InitAEforCurrentTable;
  FillAEcontrolsWithData;
end;

destructor TvAddEdit.Destroy;
var
  i: integer;
begin
  if cmbCourses.Visible then
    Courses.unRegisterComboBox(cmbCourses);
  dteDate.Destroy;
  cmbCourses.Destroy;
  lblDate.Destroy;
  lblCourse.Destroy;
  btnNew.Destroy;
  btnGo.Destroy;
  for i := 0 to (MaxFields) do
    AE[i].Destroy;
  inherited Destroy;
end;

function TvAddEdit.IterationOfFillAEcontrolsWithData(const currField: integer; const CurrAE: integer;
  const FieldType:vFieldType):boolean;
begin
  Result := True;
  case FieldType of
    vCourse: begin
               cmbCourses.ItemIndex := cmbCourses.Items.IndexOf(Fields[currField].Value);
               CoursesOnChange(cmbCourses);
             end;
    vDate: begin
             Fields[currField].Value :=
               ifthen(Fields[currField].Value='',FormatDateTime('dd/mm/yyyy', Date),Fields[currField].Value);
             dteDate.Text         := Fields[currField].Value;
           end;
  end;
  AE[CurrAE].Text := Fields[currField].Value;
end;

Function TvAddEdit.DataChanged: boolean;
begin
  UpdateAEfromSpecialControls;
  Result := not AEiterate(@IterationOfDataChanged);
end;

function TvAddEdit.IterationOfDataChanged(const currField: integer; const CurrAE: integer; const FieldType: vFieldType): boolean;
begin
  Result := True;
  if comparetext(Fields[currField].Value,AE[CurrAE].Text) <> 0 then Result := False;
  Unused(FieldType);
end;

function TvAddEdit.IterationOfSaveData(const currField: integer; const CurrAE: integer;
  const FieldType: vFieldType): boolean;
begin
  Result := true;
  Fields[currField].Value := AE[CurrAE].Text;
  case FieldType of
    vCourse: Fields[currField].Value := Courses.CourseID(Fields[currField].Value);
    vDescription: OtherDetails[2]    := AE[CurrAE].Text;
    vDistance:    OtherDetails[0]    := AE[CurrAE].Text;
    vURL:         OtherDetails[1]    := AE[CurrAE].Text;
  end;
end;

function TvAddEdit.IterationOfValidateData(const currField: integer;
  const CurrAE: integer; const FieldType: vFieldType): boolean;
var
  ErrorStr: string = '';
begin
  Result := True;
  case FieldType of
    vDate:    Result := ValidateDate(ErrorStr, dteDate.Text);
    vSeconds: Result := ValidateSeconds(ErrorStr, AE[CurrAE].Text);
  end;
  ErrorMsg := ifthen(ErrorStr = '', ErrorMsg, ErrorMsg + Errorstr + LineEnding);
  Unused(currField);
end;

function TvAddEdit.IterationOfCoursesOnChange(const currField: integer;
  const CurrAE: integer; const FieldType: vFieldType): boolean;
begin
  Result := True;
  case FieldType of
    vDescription: SetText(CurrAE,Courses.CourseDescription(cmbCourses.Text), True);
    vDistance:    SetText(CurrAE,Courses.CourseDistance(cmbCourses.Text), True);
    vURL:         SetText(CurrAE,Courses.CourseURL(cmbCourses.Text), True);
  end;
  Unused(currField);
end;

procedure TvAddEdit.FillAEcontrolsWithData;
begin
  AEiterate(@IterationOfFillAEcontrolsWithData);
end;

function TvAddEdit.SaveData(out ErrorMessage: string):boolean;
begin
  Result   := True;
  if not DataChanged then exit;
  AEiterate(@IterationOfSaveData);
  if not ValidData then begin
    ErrorMessage := ErrorMsg;
    Result := False;
    exit;
  end;
  StringData.SaveData(Fields,CurrentTable);
end;

function TvAddEdit.ValidData: boolean;
begin
  Result := True;
  ErrorMsg := 'The data cannot be saved. It is not valid:' + LineEnding;
  Result := AEiterate(@IterationOfValidateData);
  ErrorMsg := ErrorMsg + LineEnding + 'Please fix the errors and try to save again.';
end;

procedure TvAddEdit.btnGoClick(Sender: TObject);
begin
  if AE[btnGo.Tag].Text <> '' then ShowMap(AE[btnGo.Tag].Text);
end;

function TvAddEdit.GetCourseName(out OtherInfo:TvUpdateCourseType): string;
begin
  Result := AE[0].Text;
  OtherInfo := OtherDetails;
end;

procedure TvAddEdit.UpdateAEfromSpecialControls;
begin
  if dteDate.Visible then
    AE[dteDate.Tag].Text := dteDate.Text;
  if cmbCourses.Visible then
    AE[cmbCourses.Tag].Text := cmbCourses.Text;
end;

procedure TvAddEdit.btnNewClick(Sender: Tobject);
var
  frmAddEditGUI : TfrmAddEditGUI;
  Feelds: vFieldArrayType;
  i: integer;
  OtherInfo: TvUpdateCourseType;
begin
  for i := 0 to MaxFields do Feelds[i].Value:='';
  frmAddEditGUI := TfrmAddEditGUI.Create(Application, TableData, Courses, StringData, Feelds, Courses.TableName);
  try
    if frmAddEditGUI.ShowModal = mrOk then begin
      cmbCourses.ItemIndex := cmbCourses.Items.IndexOf(frmAddEditGUI.GetCourseName(OtherInfo));
      cmbCourses.Hint      := OtherInfo[2];
      AE[1].Text:=OtherInfo[0];
      AE[6].Text:=OtherInfo[2];
    end;
  finally
    frmAddEditGUI.Destroy;
  end;
  frmAddEditGUI := nil;
end;

function TvAddEdit.ValidateDate(out ErrorMessage: string; const Text: string): boolean;
var
  Valid: boolean = True;
begin
  if length(Text) <> 10 then Valid := False;
  if (PosEx('/',Text) <> 3) or (PosEx('/',Text,4) <> 6) then Valid := False;
  if (Numb2Dec(MidStr(Text,1,2),10) > 31) or (Numb2Dec(MidStr(Text,4,2),10) > 12) or
    (Numb2Dec(MidStr(Text,7,4),10) < 1900) or (Numb2Dec(MidStr(Text,7,4),10) > 2200) then
      Valid := False;
  if not Valid then
    ErrorMessage := 'The date is invalid (dd/mm/yyyy).';
  Result := Valid;
end;

function TvAddEdit.ValidateSeconds(out ErrorMessage: string; const Text: string
  ): boolean;
var
  Valid: boolean = True;
  colon1:  integer = 0;
  colon2:  integer = 0;
  mins, secs: string;
begin
  secs   := '0';
  mins   := '0';
  colon1 := npos(':', Text, 1);
  colon2 := npos(':', Text, 2);
  if colon1 < 1 then
    Mins := Text
  else if colon2 < 1 then
  begin
    mins := copy(Text, 1, colon1 - 1);
    secs := copy(Text, colon1 + 1, 10);
  end
  else begin
    mins := copy(Text,colon1+1,colon2-colon1-1);
    secs := copy(Text, colon2 + 1, 10);
  end;
  if (StrToIntdef(mins,0) > 59) or (StrToIntDef(secs,0) > 59) then Valid := False;
  if TrimLeftSet(Text,['0','1','2','3','4','5','6','7','8','9',':']) <> '' then Valid := False;
  if not Valid then
    ErrorMessage := 'The format of minutes and seconds is invalid (mm:ss).';
  Result := Valid;
end;

procedure TvAddEdit.CoursesOnChange(Sender: Tobject);
begin
  cmbCourses.Hint := Courses.CourseDescription(cmbCourses.Text);
  AEiterate(@IterationOfCoursesOnChange);
end;

function TvAddEdit.AEiterate(AEoperation: TvAEoperation):boolean;
var
  i:          integer = 1;
  currAE:     integer = 0;
  currField:  integer = 1;
  FieldType : vFieldType;
begin
  Result := True;
  while (i <= NumberOfAEcontrols) do begin
    if not TableData.GetFieldHidden(currField, CurrentTable) then begin
      FieldType   := TableData.GetFieldType(currField, CurrentTable);
      CurrAE := SetDescriptionAEcontrol(CurrAE,currField,FieldType);
      if not AEoperation(currField,CurrAE,FieldType) then begin
        Result := False;
        exit;
      end;
      CurrAE := CurrAE + 1;
      i := i + 1;
    end;
    currField := CurrField + 1;
  end;
end;

function TvAddEdit.GetText(i: integer):string;
begin
  Result := AE[i].Text;
end;

procedure TvAddEdit.SetText(i: integer; Text: string; OnlyIfEmpty: boolean);
begin
  if OnlyIfEmpty and (GetText(i) <> '') then exit;
  AE[i].Text := Text;
end;

end.

