unit vTypes;

{$mode objfpc}{$H+}

///////////////////////////////////////////////////////////////
////           Unit to provide Type definitions            ////
///////////////////////////////////////////////////////////////

interface

uses
  Classes, SysUtils;

const
  MaxCourses = 100;
  MaxFields  = 12;//10;
  MaxTables  = 10;

type

  // Type containing Course metadata
  VCourseDataType = record
    ID, Name, Distance, URL, Description: string;
  end;

  VCourseArrayType = array [0..MaxCourses] of VCourseDataType;

  // Type containing Field Names or Values
  VFieldNVType = record
    Value: string;
    Name: string;
  end;

  // Array for names of fields in currently used table.
  VFieldArrayType = array [0..MaxFields] of VFieldNVType;

  // Table's Field type. When changing this, also change SectionNames below
  VFieldType = (vOther, vString, vDescription, vDate, vSeconds,
    vDistance, vURL, vCourse);
  AllFieldTypes = vString..vCourse;

  // Type containing metadata on the fields in each table
  VFieldInfoType = record
    FN: string;                 // Field name
    UFN: string;                // User-friendly name for the field
    DefaultValue: String;       // Default value for adding a new record.
    Hidden: boolean;            // True = do not display this field
    FT: VFieldType;             // Field type (eg string, course, seconds)
    Y: boolean;                 // The default Y value for the graph
    X: boolean;                 // The default X value for the graph
    NZC: integer;               // Counter of all non-zero entries for this field
    Tot, Min, Max: real;        // Min, max values for this field, and a total summation.
  end;

  VFieldInfoArrayType = array [0..MaxFields] of VFieldInfoType;

  // Type containing metadata on the tables
  VTableInfoType = record
    TN: string;                         // Table name
    UFN: string;                        // User-friendly name for the table
    NF: integer;                        // Number of Fields
    Hidden: boolean;                    // True = do not display
    NotGraphed: boolean;                // True = do not graph this table
    FieldInfo: VFieldInfoArrayType;     // Metadata on each field in the table
    Count: integer;                     // Total of all records in the table.
  end;

  VTableInfoArrayType = array [0..MaxTables] of VTableInfoType;

  //  A record used for returning a Filter Row
  VfilterRow = record
    Quantity:   string;
    Field:      string;
    Comparison: string;
    Course:     integer;
    DateFrom:   string;
    DateTo:     string;
  end;

  // Procedure type called by TvStringData to redraw graph in TvGraphs
  TvDrawGraph = procedure (const ResetYvalues: boolean = True) of object;

    // Options controlling the Filter's behaviour
    TvFilterOptions = record
      StickyCourse, StickyDates: boolean;
    end;

const

  SectionNames: array[AllFieldTypes] of string = ('StringField', 'DescriptionField', 'DateField', 'SecondsField',
       'DistanceField', 'URLfield', 'CourseField');

implementation

end.

