unit vUtils;

{$mode objfpc}{$H+}

interface

uses
  SysUtils, LazHelpHTML, UTF8Process, Graphics, GraphType, FileUtil;

// Format the date from yyyymmdd to dd/mm/yyyy or vice versa
function FormatDate(dte: string; ForDisplay: boolean = False): string;
// Format the seconds as minutes:seconds or vice versa
function FormatSeconds(sec: string; ForDisplay: boolean=False): string;
// Open the Browser and the URL (a map hopefully)
procedure ShowMap(URL: string);
function PickColour: TGraphicsColor;
procedure BackupDB(path: string);
// Call this procedure to avoid 'unused parameter' compiler warnings
procedure Unused(const A1);
procedure Unused(const A1, A2);

implementation

uses
  StrUtils;

function FormatDate(dte: string; ForDisplay: boolean = False): string;
begin
  case ForDisplay of
    True:  Result := copy(dte, 7, 2) + '/' + copy(dte, 5, 2) + '/20' + copy(dte, 3, 2);
    False: Result := '20' + RightStr(dte, 2) + copy(dte, 4, 2) + copy(dte, 1, 2);
  end;
end;

function FormatSeconds(sec: string; ForDisplay: boolean = False): string;
var
  totsecs: integer = 0;
  totmins: integer = 0;
  tothrs:  integer = 0;
  colon1:  integer = 0;
  colon2:  integer = 0;
  hrs, mins, secs: string;
begin
  case ForDisplay of
    True:
    begin
      totsecs := StrToIntDef(sec, 0);
      secs    := IntToStr(totsecs mod 60);
      totmins := totsecs div 60;
      if totmins > 60 then begin
        tothrs  := totmins div 60;
        totmins := totmins mod 60;
      end;
      mins := IntToStr(totmins);
      hrs  := IntToStr(tothrs);
      if Length(secs) = 1 then
        secs := '0' + secs;
      if (tothrs > 0) and (Length(mins) = 1) then
        mins := '0' + mins;
      if tothrs = 0 then
        Result := ifthen(totsecs = 0,'', mins + ':' + secs)
      else
        Result := hrs + ':' + mins + ':' + secs;
    end;
    False:
    begin
      secs   := '0';
      mins   := '0';
      hrs    := '0';
      colon1 := npos(':', sec, 1);
      colon2 := npos(':', sec, 2);
      if colon1 < 1 then
        Mins := sec
      else if colon2 < 1 then
      begin
        mins := copy(sec, 1, colon1 - 1);
        secs := copy(sec, colon1 + 1, 10);
      end
      else begin
        hrs  := copy(sec, 1, colon1 - 1);
        mins := copy(sec,colon1+1,colon2-colon1-1);
        secs := copy(sec, colon2 + 1, 10);
      end;
      Result := IntToStr(StrToIntDef(hrs,0)*3600 + StrToIntDef(mins,0) * 60 + StrToIntDef(secs,0));
    end;
  end;
end;

procedure ShowMap(URL: string);
var
  v: THTMLBrowserHelpViewer;
  BrowserPath, BrowserParams: string;
  p: longint;
  BrowserProcess: TProcessUTF8;
begin
  v := THTMLBrowserHelpViewer.Create(nil);
  try
    v.FindDefaultBrowser(BrowserPath, BrowserParams);
    // Replace %s with the URL
    p := System.Pos('%s', BrowserParams);
    System.Delete(BrowserParams, p, 2);
    System.Insert(URL, BrowserParams, p);
    // start browser
    BrowserProcess := TProcessUTF8.Create(nil);
    try
      BrowserProcess.CommandLine := BrowserPath + ' ' + BrowserParams;
      BrowserProcess.Execute;
    finally
      BrowserProcess.Free;
    end;
  finally
    v.Free;
  end;
end;

function PickColour: TGraphicsColor;
begin
  Result := RGBToColor(Random(256), Random(256), Random(256));
end;

procedure BackupDB(path: string);
begin
  CopyFile(PChar(path + '/VfitnessDB.1'), PChar(path + '/VfitnessDB.2'), False);
  CopyFile(PChar(path + '/VfitnessDB'), PChar(path + '/VfitnessDB.1'), False);
  CopyFile(PChar(path + '/Vfitness1.ini'), PChar(path + '/Vfitness2.ini'), False);
  CopyFile(PChar(path + '/Vfitness.ini'), PChar(path + '/Vfitness.ini.1'), False);
  CopyFile(PChar(path + '/filters.ini.1'), PChar(path + '/filters.ini.2'), False);
  CopyFile(PChar(path + '/filters.ini'), PChar(path + '/filters.ini.1'), False);
  CopyFile(PChar(path + '/defaults.ini.1'), PChar(path + '/defaults.ini.2'), False);
  CopyFile(PChar(path + '/defaults.ini'), PChar(path + '/defaults.ini.1'), False);
end;

{$HINTS OFF}
procedure Unused(const A1);
begin
end;

procedure Unused(const A1, A2);
begin
end;
{$HINTS ON}

end.

